/*================================================================
  decode.h
  used with spectrum.cpp
  calls instructions from the current PC
  written by z80man
  ================================================================*/
  
int decode(void)
{
	unsigned int instruction = readmemory_word(PC);     //load instruction from memory
	
	half_byte1:4 = (instruction & 0x0F00) / 0x100;
	half_byte2:4 = (instruction & 0x00F0) / 0x10;
	
	low_byte = instruction & 0x00FF;   
	
	switch(instruction)
	{
	case instruction & ADD_v: //adds Rn and Rm into Rn: ADD Rm,Rn
		ADD();
		break;
	case instruction & MOVLi_v: //copies data at disp*4 into Rn: MOV.L @(disp,PC),Rn
		MOVLi();
		break;
	case instruction & MOVr_v: //copies data from Rm into Rn: MOV Rm,Rn
		MOVr();
		break;
	case instruction & MOVLin_v: //copies data from Rm into adress of Rn: MOV.L Rm,@Rn  
		MOVLin();
		break;
	case instruction & MOVLim_v: //copies data from adress of Rm into Rn: MOV.L @Rm,Rn
		MOVLim();
		break;
	case instruction & MOVLpush_v: //copies data from Rm into adress Rn. Rn decrements by four: MOV.L RM,@-Rn
		MOVLpush();
		break;
	case instruction & MOVLpop_v: //copies data from adress of Rm into Rn. Rm increments by four: MOV.L @Rm+,Rn
		MOVLpop();
		break;
	case instruction & CMPEQr_v: //if Rm is equal to Rn, then T equals 1: CMP/EQ Rm,Rn
		CMPEQr();
		break;
	case instruction & DT_v: //Rn decrements by 1. If Rn equals 0, then 1 equals T
		DT();
		break;
	case instruction & SUB_v: //subtracts Rm from Rn: SUB Rm,Rn
		SUB();
		break;
	case instruction & ANDr_v: //binary and of Rm and Rn into Rn: AND Rm,Rn
		ANDr();
		break;
	case instruction & NOT_v: //inverse of Rm becomes Rn: NOT Rm,Rn
		NOT();
		break;
	case instruction & ORr_v: //binary or of Rm and Rn becomes Rn: Or Rm,Rn
		OR();
		break;
	case instruction &	TSTr_v: //binary and of Rm and Rn. If result equals 0, then 1 equals T
		TSTr();
		break;
	case instruction & XORr_v: //binary xor of Rm and Rn into Rn: XOR Rm,Rn
		XORr();
		break;
	case instruction & BF_v: //if T equals zero, then disp*2 + PC equals PC: BF label
		BF()
		break;
	case instruction & BTS_v: //if T equals 1, then disp*2 plus PC equals PC: BT label
		BTS()
		break;
	case instruction & NOP_v: //no operation
		NOP();
		break;
	default:
		NOP();
	}
return;	
		